import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  ProtocolDecoder,
  ProtocolMessage,
  PacketBodyValue,
  BodyStringDivider,
  ProtocolStream
} from '@ucap-webmessenger/protocol';
import { EmployeeType } from '@ucap-webmessenger/protocol-room';
import { RoleCode } from '@ucap-webmessenger/protocol-authentication';
import { UserDnSearchType } from '../types/user-dn-search.type';

export interface UserDnRequest extends ProtocolRequest {
  // DivCD(s)
  divCd: string;
  // 기관코드(s)
  companyCode: string;
  // 부서SEQ(n)
  seq?: number;
  // 검색string(s)
  search?: string;
  // 검색구분(s)
  searchType: UserDnSearchType;
  // 요청리스트(n) 사용안하면 0 최대 200
  pageListCount?: number;
  // 요청페이지(n) 사용안하면 0 처음은 1
  pageCurrent?: number;
  // 발신자회사코드(s)
  senderCompanyCode: string;
  // 발신자임직원유형(s)
  senderEmployeeType: EmployeeType;
}

export interface UserInfoDN {
  // 사용자SEQ
  seq: number;
  // 사용자명
  name: string;
  // 사진파일
  profileImageFile: string;
  // 직급
  grade: string;
  // 업무소개
  intro: string;
  // 기관코드
  companyCode: string;
  // 핸드폰번호
  hpNumber: string;
  // 내선번호
  lineNumber: string;
  // 이메일
  email: string;
  // 모바일YN
  isMobile: boolean;
  // 조회순서
  order: string;
  // ActiveYN
  isActive: boolean;
  // 역할코드
  roleCd: RoleCode;
  // 사번
  employeeNum: string;
  // 부서명
  deptName: string;
  // MADN
  madn: string;
  // HARDPHONE_SADN
  hardSadn: string;
  // FMC_SADN
  fmcSadn: string;
  // 사용자명(영어)
  nameEn: string;
  // 사용자명(중국어)
  nameCn: string;
  // 직급(영어)
  gradeEn: string;
  // 직급(중국어)
  gradeCn: string;
  // 부서명(영어)
  deptNameEn: string;
  // 부서명(중국어)
  deptNameCn: string;
}
export interface UserDnData extends ProtocolStream {
  // DivCD(s)
  divCd: string;
  // {부서정보-S}...
  userDnInfos: UserInfoDN[];
}
export interface UserDnResponse extends ProtocolResponse {
  // DivCD(s)
  divCd: string;
  // 전체갯수(n)
  pageTotalCount: number;
  // 요청리스트(n)
  pageListCount?: number;
  // 페이지(n)
  pageCurrent?: number;
}

export const encodeUserDn: ProtocolEncoder<UserDnRequest> = (
  req: UserDnRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push(
    { type: PacketBodyValue.String, value: req.divCd },
    { type: PacketBodyValue.String, value: req.companyCode },
    { type: PacketBodyValue.Integer, value: req.seq || 0 },
    { type: PacketBodyValue.String, value: req.search || '' },
    { type: PacketBodyValue.String, value: req.searchType },
    {
      type: PacketBodyValue.Integer,
      value: req.pageListCount || 0
    },
    { type: PacketBodyValue.Integer, value: req.pageCurrent || 0 },
    { type: PacketBodyValue.String, value: req.senderCompanyCode },
    {
      type: PacketBodyValue.String,
      value: req.senderEmployeeType
    }
  );

  return bodyList;
};

export const decodeUserDnData: ProtocolDecoder<UserDnData> = (
  message: ProtocolMessage
) => {
  const userDnInfos: UserInfoDN[] = [];
  message.bodyList.slice(1).forEach(userDnInfo => {
    const info = userDnInfo.split(BodyStringDivider);
    let i = 0;
    userDnInfos.push({
      seq: info[i],
      name: info[i++],
      profileImageFile: info[i++],
      grade: info[i++],
      intro: info[i++],
      companyCode: info[i++],
      hpNumber: info[i++],
      lineNumber: info[i++],
      email: info[i++],
      isMobile: info[i++] === 'Y' ? true : false,
      order: info[i++],
      isActive: info[i++] === 'Y' ? true : false,
      roleCd: info[i++] as RoleCode,
      employeeNum: info[i++],
      deptName: info[i++],
      madn: info[i++],
      hardSadn: info[i++],
      fmcSadn: info[i++],
      nameEn: info[i++],
      nameCn: info[i++],
      gradeEn: info[i++],
      gradeCn: info[i++],
      deptNameEn: info[i++],
      deptNameCn: info[i++]
    });
  });
  return {
    divCd: message.bodyList[0],
    userDnInfos
  } as UserDnData;
};

export const decodeUserDn: ProtocolDecoder<UserDnResponse> = (
  message: ProtocolMessage
) => {
  return {
    divCd: message.bodyList[0],
    pageTotalCount: message.bodyList[1],
    pageListCount: message.bodyList[2],
    pageCurrent: message.bodyList[3]
  } as UserDnResponse;
};
